from qupyt.algorithms.deutsch import deutsch
from qupyt.algorithms.deutsch_josza import deutsch_josza
from qupyt.algorithms.bernstein_vazirani import bernstein_vazirani
from qupyt.algorithms.grover import grover, animate_grover